from django import VERSION
from django.conf import settings
from django.contrib.sites.models import Site
from django.template import TemplateDoesNotExist

from dbtemplates.models import Template, backend

def load_template_source(template_name, template_dirs=None):
    """
    A custom template loader to load templates from the database.

    Tries to load the template from the dbtemplates cache backend specified
    by the DBTEMPLATES_CACHE_BACKEND setting. If it does not find a template
    it falls back to query the database field ``name`` with the template path
    and ``sites`` with the current site.
    """
    if VERSION[:2] >= (1, 2):
        # For backward compatibility
        import warnings
        warnings.warn(
            "`dbtemplates.loader.load_template_source` is deprecated; "
            "use `dbtemplates.loader.Loader` instead.",
            PendingDeprecationWarning
        )
    site = Site.objects.get_current()
    display_name = 'db:%s:%s:%s' % (settings.DATABASE_ENGINE,
                                    template_name, site.domain)
    if backend:
        try:
            backend_template = backend.load(template_name)
            if backend_template:
                return backend_template, template_name
        except:
            pass
    try:
        template = Template.on_site.get(name__exact=template_name)
        # Save in cache backend explicitly if manually deleted or invalidated
        if backend:
            backend.save(template_name, template.content)
        return (template.content, display_name)
    except:
        pass
    raise TemplateDoesNotExist(template_name)
load_template_source.is_usable = True


if VERSION[:2] >= (1, 2):
    # providing a class based loader for Django >= 1.2, yay!
    from django.template.loader import BaseLoader

    class Loader(BaseLoader):
        __doc__ = load_template_source.__doc__

        is_usable = True

        def load_template_source(self, template_name, template_dirs=None):
            return load_template_source(template_name, template_dirs)




import os
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

CACHE_BACKEND = getattr(settings, 'DBTEMPLATES_CACHE_BACKEND', None)

ADD_DEFAULT_SITE = getattr(settings, 'DBTEMPLATES_ADD_DEFAULT_SITE', True)

MEDIA_PREFIX = getattr(settings, 'DBTEMPLATES_MEDIA_PREFIX',
    os.path.join(settings.MEDIA_ROOT, 'dbtemplates'))

USE_REVERSION = getattr(settings, 'DBTEMPLATES_USE_REVERSION', False)

if USE_REVERSION and 'reversion'not in settings.INSTALLED_APPS:
    raise ImproperlyConfigured("Please add reversion to your INSTALLED_APPS setting to make use of it in dbtemplates.")

USE_CODEMIRROR = getattr(settings, 'DBTEMPLATES_USE_CODEMIRROR', False)



from django import VERSION
from django.test import TestCase
from django.contrib.sites.models import Site
from django.template import loader, Context
from django.core.management import call_command

from dbtemplates import settings
from dbtemplates.loader import load_template_source

from dbtemplates.models import Template

class DbTemplatesTestCase(TestCase):
    def setUp(self):
        self.site1, created1 = Site.objects.get_or_create(domain="example.com", name="example.com")
        self.site2, created2 = Site.objects.get_or_create(domain="example.org", name="example.org")

    def test_basiscs(self):
        t1 = Template(name='base.html', content='<html><head></head><body>{% block content %}Welcome at {{ title }}{% endblock %}</body></html>')
        t1.save()
        self.assertEqual(Site.objects.get_current(), t1.sites.all()[0])
        self.assertTrue("Welcome at" in t1.content)

        t2 = Template(name='sub.html', content='{% extends "base.html" %}{% block content %}This is {{ title }}{% endblock %}')
        t2.save()
        t2.sites.add(self.site2)

        self.assertEqual(list(Template.objects.filter(sites=self.site1)), [t1, t2])
        self.assertEqual(list(t2.sites.all()), [self.site1, self.site2])

    def test_load_templates(self):
        self.test_basiscs()
        loader.template_source_loaders = [load_template_source]
        result1 = loader.get_template("base.html").render(Context({'title':'MainPage'}))
        self.assertEqual(result1, u'<html><head></head><body>Welcome at MainPage</body></html>')

        result2 = loader.get_template("sub.html").render(Context({'title':'SubPage'}))
        self.assertEqual(result2, u'<html><head></head><body>This is SubPage</body></html>')

        if VERSION[:2] >= (1, 2):
            from dbtemplates.loader import Loader
            dbloader = Loader()
            loader.template_source_loaders = [dbloader.load_template_source]
            result = loader.get_template("base.html").render(Context({'title':'MainPage'}))
            self.assertEqual(result, u'<html><head></head><body>Welcome at MainPage</body></html>')
            result2 = loader.get_template("sub.html").render(Context({'title':'SubPage'}))
            self.assertEqual(result2, u'<html><head></head><body>This is SubPage</body></html>')

    def test_error_templates_creation(self):
        call_command('create_error_templates', force=True, verbosity=0)
        self.assertEqual(list(Template.objects.filter(sites=self.site1)),
            list(Template.objects.filter()))

    def test_disabling_default_site(self):
        old_add_default_site = settings.ADD_DEFAULT_SITE
        settings.ADD_DEFAULT_SITE = False
        t3 = Template.objects.create(name='footer.html', content='ohai')
        self.assertEqual(list(t3.sites.all()), [])
        settings.ADD_DEFAULT_SITE = old_add_default_site

    def test_automatic_sync(self):
        admin_base_template, origin = loader.find_template_source('admin/base.html')
        template = Template.objects.create(name='admin/base.html')
        self.assertEqual(admin_base_template, template.content)



import posixpath
from django import forms
from django.conf import settings
from django.contrib import admin
from django.utils.translation import ungettext, ugettext_lazy as _
from django.utils.safestring import mark_safe

from dbtemplates.settings import MEDIA_PREFIX, USE_REVERSION, USE_CODEMIRROR
from dbtemplates.models import Template, backend, remove_cached_template, \
    add_template_to_cache

# Check if django-reversion is installed and use reversions' VersionAdmin
# as the base admin class if yes
if USE_REVERSION:
    from reversion.admin import VersionAdmin as TemplateModelAdmin
else:
    from django.contrib.admin import ModelAdmin as TemplateModelAdmin

class CodeMirrorTextArea(forms.Textarea):
    """
    A custom widget for the CodeMirror browser editor to be used with the 
    content field of the Template model.
    """
    class Media:
        css = dict(screen=[posixpath.join(MEDIA_PREFIX, 'css/editor.css')])
        js = [posixpath.join(MEDIA_PREFIX, 'js/codemirror.js')]

    def render(self, name, value, attrs=None):
        result = []
        result.append(
            super(CodeMirrorTextArea, self).render(name, value, attrs))
        result.append(u"""
<script type="text/javascript">
  var editor = CodeMirror.fromTextArea('id_%(name)s', {
    path: "%(media_prefix)sjs/",
    parserfile: "parsedjango.js",
    stylesheet: "%(media_prefix)scss/django.css",
    continuousScanning: 500,
    height: "40.2em",
    tabMode: "shift",
    indentUnit: 4,
    lineNumbers: true
  });
</script>
""" % dict(media_prefix=MEDIA_PREFIX, name=name))
        return mark_safe(u"".join(result))

if USE_CODEMIRROR:
    TemplateContentTextArea = CodeMirrorTextArea
else:
    TemplateContentTextArea = forms.Textarea


class TemplateAdminForm(forms.ModelForm):
    """
    Custom AdminForm to make the content textarea wider.
    """
    content = forms.CharField(
        widget=TemplateContentTextArea({'rows': '24'}),
        help_text=_("Leaving this empty causes Django to look for a template "
            "with the given name and populate this field with its content."),
        required=False)

    class Meta:
        model = Template


class TemplateAdmin(TemplateModelAdmin):
    form = TemplateAdminForm
    fieldsets = (
        (None, {
            'fields': ('name', 'content'),
            'classes': ('monospace',),
        }),
        (_('Advanced'), {
            'fields': (('sites'),),
            'classes': ('collapse',),
        }),
        (_('Date/time'), {
            'fields': (('creation_date', 'last_changed'),),
            'classes': ('collapse',),
        }),
    )
    list_display = ('name', 'creation_date', 'last_changed', 'site_list')
    list_filter = ('sites',)
    search_fields = ('name', 'content')
    if backend:
        actions = ['invalidate_cache', 'repopulate_cache']

    def invalidate_cache(self, request, queryset):
        if not backend:
            self.message_user(request, ("There is no active cache backend."))
            return
        for template in queryset:
            remove_cached_template(template)
        message = ungettext(
            "Cache of one template successfully invalidated.",
            "Cache of %(count)d templates successfully invalidated.",
            len(queryset))
        self.message_user(request, message % {'count': len(queryset)})
    invalidate_cache.short_description = _("Invalidate cache of selected templates")

    def repopulate_cache(self, request, queryset):
        if not backend:
            self.message_user(request, ("There is no active cache backend."))
            return
        for template in queryset:
            add_template_to_cache(template)
        message = ungettext(
            "Cache successfully repopulated with one template.",
            "Cache successfully repopulated with %(count)d templates.",
            len(queryset))
        self.message_user(request, message % {'count': len(queryset)})
    repopulate_cache.short_description = _("Repopulate cache with selected templates")

    def site_list(self, template):
          return ", ".join([site.name for site in template.sites.all()])
    site_list.short_description = _('sites')

admin.site.register(Template, TemplateAdmin)
